@@ -4,38 +4,16 @@
 # 2012-04-04: modified to allow weighted linear models. J. Fox
 # 2020-06-25: Fix bug in hccm.lm() when model matrix includes just one column 
 #             (reported by Justin Yap). J. Fox
+# 2021-07-29: Report error when any hatvalue = 1 for all but hc0 and hc1
+#             (following report of problem reported by Peng Ding) J. Fox
 #-------------------------------------------------------------------------------
 
-# Heteroscedasticity-corrected standard errors (White adjustment) (J. Fox)
+# Heteroscedasticity-corrected standard errors (Huber/White adjustment) (J. Fox)
 
 hccm <- function(model, ...){
-	#last modified 12 Dec 2000 by J. Fox
 	UseMethod("hccm")
 }
 
-#hccm.lm <- function (model, type = c("hc3", "hc0", "hc1", "hc2", "hc4"), 
-#		singular.ok=TRUE, ...) {
-#	if (!is.null(weights(model))) 
-#		stop("requires unweighted lm")
-#	type <- match.arg(type)
-#	if (any(aliased <- is.na(coef(model))) && !singular.ok)
-#		stop("there are aliased coefficients in the model")
-#	sumry <- summary(model, corr = FALSE)
-#	s2 <- sumry$sigma^2
-#	V <- sumry$cov.unscaled
-#	if (type == FALSE) 
-#		return(s2 * V)
-#	e <- na.omit(residuals(model))
-#	X <- model.matrix(model)[, !aliased]
-#	df.res <- df.residual(model)
-#	n <- length(e)
-#	h <- hat(X)
-#	p <- ncol(X)
-#	factor <- switch(type, hc0 = 1, hc1 = df.res/n, hc2 = 1 - 
-#					h, hc3 = (1 - h)^2, hc4 = (1 - h)^pmin(4, n * h/p))
-#	V %*% t(X) %*% apply(X, 2, "*", (e^2)/factor) %*% V
-#}
-
 hccm.lm <-function (model, type = c("hc3", "hc0", "hc1", "hc2", "hc4"), 
 		singular.ok = TRUE, ...) {
 	e <- na.omit(residuals(model))
@@ -60,12 +38,29 @@ hccm.lm <-function (model, type = c("hc3", "hc0", "hc1", "hc2", "hc4"),
 	n <- length(e)
 	e <- wts*e
 	p <- ncol(X)
-	factor <- switch(type, hc0 = 1, hc1 = df.res/n, 
+	factor <- switch(type, hc0 = 1, hc1 = df.res/n,
 			hc2 = 1 - h, hc3 = (1 - h)^2, hc4 = (1 - h)^pmin(4, n * h/p))
-	V %*% t(X) %*% apply(X, 2, "*", (e^2)/factor) %*% V
+	V <- V %*% t(X) %*% apply(X, 2, "*", (e^2)/factor) %*% V
+	bad <- h > 1 - sqrt(.Machine$double.eps)
+	if ((n.bad <- sum(bad)) > 0 && !(type %in% c("hc0", "hc1"))) {
+	  nms <- names(e)
+	  bads <- if (n.bad <= 10) {
+	    paste(nms[bad], collapse=", ")
+	  } else {
+	    paste0(paste(nms[bad[1:10]], collapse=", "), ", ...")
+	  }
+	  if (any(is.nan(V))){
+	    stop("cannot proceed because of ", n.bad, if (n.bad == 1) " case " else " cases ",
+	       "with hatvalue = 1:\n   ", bads)
+	  } else {
+	    warning("adjusted coefficient covariances may be unstable because of ", n.bad, 
+	            if (n.bad == 1) " case " else " cases ",
+	            "with hatvalue near 1:\n   ", bads)
+	  }
+	}
+	V
 }
 
 hccm.default<-function(model, ...){
-	#last modified 12 Dec 2000 by J. Fox
 	stop("requires an lm object")
 }
